import os
from importlib.resources import files

import sass
import streamlit as st

from .config import get_config_value
from .. import ConfigOptions, STYLE_SASS_MAIN_FILE_NAME


def run_compile():
    # Define the input SASS file
    sass_path = get_config_value(ConfigOptions.SASS_SOURCE_PATH.value)
    input_file = os.path.join(sass_path, STYLE_SASS_MAIN_FILE_NAME)
    include_paths = [str(files(f"streamlit_sal.sass")), sass_path]

    # Define the output CSS file
    css_path = get_config_value(ConfigOptions.CSS_STYLESHEET_FILE_PATH.value)
    css_file = get_config_value(ConfigOptions.CSS_STYLESHEET_FILE_NAME.value)
    output_file = os.path.join(css_path, css_file)

    with open(output_file, 'w') as f:
        compiled_css = sass.compile(filename=input_file,
                                    include_paths=include_paths)  # TODO Consider using output_style='compressed' ?
        variables_css = generate_st_theme_variables_css()
        f.write(variables_css + '\n' + compiled_css)
        print(f"Compiled {input_file} to {output_file}")


def generate_st_theme_variables_css():
    primary_color = st.get_option('theme.primaryColor')
    background_color = st.get_option('theme.backgroundColor')
    secondary_background_color = st.get_option('theme.secondaryBackgroundColor')
    text_color = st.get_option('theme.textColor')
    font = st.get_option('theme.font')
    base = st.get_option('theme.base')

    css_vars = [
        f'--st-primary-color: {primary_color};' if primary_color is not None else '',
        f'--st-background-color: {background_color};' if background_color is not None else '',
        f'--st-secondary-background-color: {secondary_background_color};' if secondary_background_color is not None else '',
        f'--st-text-color: {text_color};' if text_color is not None else '',
        f'--st-font: {font};' if font is not None else '',
        f'--st-theme-base: {base};' if base is not None else ''
    ]

    # Ignore any empty string vars
    css_vars_no_empty_str = [var for var in css_vars if var]

    if len(css_vars_no_empty_str) > 0:
        # Join the variables with a newline character
        css_vars_str = "\n  ".join(css_vars_no_empty_str)

        return (
            ":root {\n"
            f"  {css_vars_str}\n"
            "}"
        )

    return ''



import os
from configparser import ConfigParser
from .. import CONFIG_FILE_NAME, CONFIG_DEFAULT_SECTION, ConfigOptions


def read_config():
    config = ConfigParser()
    config_file_path = find_config_file()
    config.read(config_file_path)
    return config


def update_config(key, value):
    config = read_config()
    config.set(CONFIG_DEFAULT_SECTION, key, value)
    save_config(config)


def save_config(config):
    config_file_path = find_config_file()
    with open(config_file_path, 'w') as configfile:
        config.write(configfile)


def get_config_value(key):
    config = read_config()
    return config.get(CONFIG_DEFAULT_SECTION, key)


def find_root_dir():
    current_dir = os.getcwd()
    while True:
        config_file_path = os.path.join(current_dir, CONFIG_FILE_NAME)
        if os.path.exists(config_file_path):
            return current_dir
        parent_dir = os.path.dirname(current_dir)
        if parent_dir == current_dir:
            # We have reached the root directory
            break
        current_dir = parent_dir
    raise Exception(f"Could not find root directory. Did you run 'streamlit-sal init'?")


def find_config_file():
    root_dir = find_root_dir()
    config_file_path = os.path.join(root_dir, CONFIG_FILE_NAME)
    if os.path.exists(config_file_path):
        return config_file_path
    else:
        raise Exception(f"Could not locate {CONFIG_FILE_NAME}. Did you run 'streamlit-sal init'?")


def get_css_filepath():
    root_dir = find_root_dir()
    path = get_config_value(ConfigOptions.CSS_STYLESHEET_FILE_PATH.value)
    name = get_config_value(ConfigOptions.CSS_STYLESHEET_FILE_NAME.value)
    css_file_path = os.path.join(root_dir, path, name)
    return css_file_path



from enum import Enum

# Stylesheet files (input/output)
STYLE_DEFAULT_DIST_DIRECTORY = '.'

STYLE_SASS_MAIN_FILE_NAME = 'main.scss'
STYLE_SASS_LAYOUT_FILE_NAME = '_layout.scss'
STYLE_SASS_UTILITIES_FILE_NAME = '_utilities.scss'
STYLE_SASS_BUILDER_FILE_NAME = '_style-builder.scss'
STYLE_SASS_ST_MAP_FILE_NAME = '_streamlit-component-map.scss'
STYLE_SASS_SOURCE_FILES = [STYLE_SASS_MAIN_FILE_NAME, STYLE_SASS_LAYOUT_FILE_NAME, STYLE_SASS_UTILITIES_FILE_NAME,
                           STYLE_SASS_BUILDER_FILE_NAME, STYLE_SASS_ST_MAP_FILE_NAME]

STYLE_DEFAULT_CSS_FILE_NAME = 'sal-stylesheet.css'

# Config util
CONFIG_FILE_NAME = '.streamlit_sal'
CONFIG_DEFAULT_SECTION = 'settings'


class ConfigOptions(Enum):
    SASS_SOURCE_PATH = 'sass_source_path'
    CSS_STYLESHEET_FILE_PATH = 'css_stylesheet_file_path'
    CSS_STYLESHEET_FILE_NAME = 'css_stylesheet_file_name'




from contextlib import contextmanager

import streamlit as st

from .config import get_css_filepath


@contextmanager
def sal_stylesheet(move_sidebar_right=False, reduce_main_container_space=False):
    css_filepath = get_css_filepath()
    try:
        with open(css_filepath, 'r') as css_file:
            css_stylesheet = css_file.read()

            if css_stylesheet:
                st.markdown(f"""
                    <style class="hidden">
                        {css_stylesheet}
                    </style>
                """, unsafe_allow_html=True)

                app_styles = []
                if reduce_main_container_space:
                    app_styles.append('sal-reduce-main-container-space')

                if move_sidebar_right:
                    app_styles.append('sal-move-sidebar-right')

                if len(app_styles) > 0:
                    st.markdown(f"<span class='{' '.join(app_styles)} hidden'></span>", unsafe_allow_html=True)
            yield
    except FileNotFoundError:
        raise FileNotFoundError(f"Could not locate stylesheet: {css_filepath}")


@contextmanager
def create_markdown_container(*args, **kwargs):
    is_hidden = kwargs.pop('is_hidden', True)
    container = kwargs.pop('container') if 'container' in kwargs else None
    component_name = kwargs.pop('component_name') if 'component_name' in kwargs else None
    if component_name is None:
        raise ValueError(f"component_name has to be defined.")

    # Replace any name underscores with dash to match class naming conventions in the style placeholders
    component_name = component_name.replace("_", "-")
    classes = list(args) if args else []
    classes.append(f"sal-{component_name}")
    if is_hidden:
        classes.append('hidden')
    if container:
        container.markdown(f"<span class='{' '.join(classes)}'></span>", unsafe_allow_html=True)
    else:
        st.markdown(f"<span class='{' '.join(classes)}'></span>", unsafe_allow_html=True)
    yield



import click
from click import ClickException

from ..utils import run_compile
from .init_files import run_init


@click.group()
def cli():
    pass


@cli.command()
def compile():
    run_compile()


@cli.command()
def init():
    if not click.confirm(text='Did you run this command from the project root directory?'):
        raise ClickException("Please run 'streamlit-sal init' from the project root directory")

    run_init()


if __name__ == "__main__":
    cli()



from .utils import create_markdown_container


def validate_container_kwarg(component_name, **kwargs):
    if 'container' not in kwargs:
        print(
            f"[INFO] Pass the layout container from your st.{component_name} call to SAL. Read more here: https://github.com/datarobot/streamlit-sal/blob/main/README.md")
        raise TypeError(f"(sal.{component_name}) Missing required keyword argument: 'container'")
    pass


# Text elements
def write(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='write')


def title(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='title')


def header(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='header')


def subheader(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='subheader')


def markdown(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='markdown')


# Formatted text elements
def caption(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='caption')


def code(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='code')


def divider(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='divider')


def latex(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='latex')


def text(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='text')


# Data elements
def dataframe(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='dataframe')


def data_editor(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='data_editor')


def table(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='table')


def metric(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='metric')


def json(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='json')


# Chart elements
def area_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='area_chart')


def bar_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='bar_chart')


def line_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='line_chart')


def map(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='map')


def altair_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='altair_chart')


def scatter_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='scatter_chart')


def bokeh_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='bokeh_chart')


def graphviz_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='graphviz_chart')


def plotly_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='plotly_chart')


def pydeck_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='pydeck_chart')


def pyplot(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='pyplot')


def vega_lite_chart(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='vega_lite_chart')


# Input elements

def button(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='button')


def download_button(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='download_button')


def form_submit_button(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='form_submit_button')


def link_button(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='link_button')


def page_link(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='page_link')


def checkbox(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='checkbox')


def color_picker(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='color_picker')


def multiselect(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='multiselect')


def radio(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='radio')


def selectbox(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='selectbox')


def select_slider(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='select_slider')


def toggle(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='toggle')


def number_input(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='number_input')


def slider(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='slider')


def date_input(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='date_input')


def time_input(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='time_input')


def text_area(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='text_area')


def text_input(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='text_input')


def camera_input(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='camera_input')


def file_uploader(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='file_uploader')


# Media elements


def audio(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='audio')


def image(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='image')


def logo(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='logo')


def video(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='video')


# Layout and container elements
def columns(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='columns')


def column(*args, **kwargs):
    # Column element has to pass its container for the span identifier
    validate_container_kwarg('column', **kwargs)
    return create_markdown_container(*args, **kwargs, component_name='column')


def container(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='container')


def dialog(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='dialog')


def empty(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='empty')


def expander(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='expander')


def form(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='form')


def popover(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='popover')


def popover_content(*args, **kwargs):
    # Popover element has to pass its container for the span identifier
    validate_container_kwarg('popover_content', **kwargs)
    return create_markdown_container(*args, **kwargs, component_name='popover_content')


def sidebar(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='sidebar')


def tabs(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='tabs')


# Chat elements
def chat_input(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='chat_input')


def chat_message(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='chat_message')


# Status elements
def success(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='success')


def info(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='info')


def warning(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='warning')


def error(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='error')


def exception(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='exception')


def progress(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='progress')


def spinner(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='spinner')


def status(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='status')


def toast(*args, **kwargs):
    return create_markdown_container(*args, **kwargs, component_name='toast')



import os
import shutil
from importlib.resources import files

import click

from .. import CONFIG_FILE_NAME, STYLE_DEFAULT_CSS_FILE_NAME, \
    STYLE_DEFAULT_DIST_DIRECTORY, ConfigOptions, STYLE_SASS_MAIN_FILE_NAME
from ..utils import update_config, get_config_value

SASS_PATH_MSG = "Enter a destination path for the SASS source main file"
CSS_PATH_MSG = "Enter a destination path for the compiled CSS file output"
CSS_FILE_MSG = "Enter a file name for the compiled CSS"


def check_css_extension(file_name):
    if file_name and '.css' not in file_name:
        return f"{file_name}.css"
    else:
        return file_name


def run_init():
    root_dir = os.getcwd()

    sass_src_path = click.prompt(text=SASS_PATH_MSG, default=STYLE_DEFAULT_DIST_DIRECTORY,
                                 type=click.Path(exists=False, file_okay=False))

    path_default = sass_src_path if sass_src_path else STYLE_DEFAULT_DIST_DIRECTORY
    css_stylesheet_path = click.prompt(text=CSS_PATH_MSG, default=path_default,
                                       type=click.Path(exists=False, file_okay=False))
    css_stylesheet_file_name = click.prompt(text=CSS_FILE_MSG, default=STYLE_DEFAULT_CSS_FILE_NAME,
                                            value_proc=check_css_extension,
                                            type=click.STRING)

    init_files(root_dir, sass_src_path, css_stylesheet_path, css_stylesheet_file_name)


def init_files(root_dir, sass_src_path=None, css_stylesheet_path=None, css_stylesheet_file_name=None):
    if not os.path.exists(CONFIG_FILE_NAME):
        copy_template_file(CONFIG_FILE_NAME, root_dir)

    if sass_src_path:
        update_config(ConfigOptions.SASS_SOURCE_PATH.value, sass_src_path)

    if css_stylesheet_path:
        update_config(ConfigOptions.CSS_STYLESHEET_FILE_PATH.value, css_stylesheet_path)

    if css_stylesheet_file_name:
        update_config(ConfigOptions.CSS_STYLESHEET_FILE_NAME.value, css_stylesheet_file_name)

    sass_source_path = get_config_value(ConfigOptions.SASS_SOURCE_PATH.value)
    if sass_source_path != root_dir:
        create_directory_if_not_exists(sass_source_path)

    # Copy the main sass file to the given or default source path
    copy_template_file(STYLE_SASS_MAIN_FILE_NAME, sass_source_path)


def copy_template_file(file_name, destination_path):
    source_file = str(files(f"streamlit_sal.templates").joinpath(file_name))
    shutil.copy(source_file, destination_path)
    click.echo(f"Copied {source_file} to {destination_path}.")


def create_directory_if_not_exists(directory_path):
    if not os.path.exists(directory_path):
        os.makedirs(directory_path)
        click.echo(f"Directory {directory_path} created.")
    else:
        click.echo(f"Directory {directory_path} already exists.")
